from nave import *
from random import randint

class Scacchiera:
	def __init__(self, dimX, dimY, numNavi):
		self.navi = [] 		# lista di navi
		self.dimX = dimX
		self.dimY = dimY
		mindim = min(dimX, dimY)
		self.nNavi = numNavi

		# se true, disegna() mostra le navi sulla scacchiera
		self.naviVisibili = False

		if numNavi > mindim:
			print("Il numero di navi e` stato ridotto a", mindim)
			self.nNavi = mindim
		self.acqua = []

	# Genero navi di dimensione a partire da min(dimX, dimY)/2 (per evitare che debordino..)
	# Ogni due navi che genero, diminuisco la dimensione di uno.
	# In una scacchiera di 8x8 con 6 navi avro` quindi:
	# 2 navi da 4, 2 navi da 3 e 2 navi da 2 caselline..
	#
	def popolaScacchiera(self):
		self.navi = [] # inizializzo lista navi 
		numeroNavi = 0 # contatore numero navi
		# una nave  puo' essere grande al massimo un numero di celle pari alla
		# meta' della dimensione della scacchiera.
		dimensioneNave = min(self.dimX, self.dimY) // 2

		while numeroNavi < self.nNavi and dimensioneNave > 1:

			collisione = False
			# genero casualmente posizione x e y e orientazione della nave
			x = randint(0, self.dimX - 1)
			y = randint(0, self.dimY - 1)
			# orientazione
			h = randint(0, 1)
			# fattore moltiplicativo 1 o -1 per generare una nave con
			# diversa posizione della prua (vedi Nave)
			fattore = 1

			if h == 0:
				orientation = "Horizontal"
				if y > self.dimX // 2:
					fattore = -1	
			else:
				orientation = "Vertical"
				if x > self.dimY // 2:
					fattore = -1
			
			print("Creo nave" , x, y, fattore * dimensioneNave, orientation)
			nave = Nave(x, y, fattore * dimensioneNave, orientation)

			for n in self.navi:
				if n.collide(nave):
					collisione = True
					print("La nave", n.pos, "collide con", nave.pos)
					break

			if not collisione:
				self.navi.append(nave)
				numeroNavi = numeroNavi + 1
				# ogni due navi, diminuisco la dimensione
				if numeroNavi % 2 == 0:
					dimensioneNave = dimensioneNave - 1

		if dimensioneNave < 2:
			print("Il numero di navi e' stato ridotto a", numeroNavi)

	# questa funzione disegna ciascuna cella della scacchiera, controllando se 
	# essa e' occupata da una nave (nave.pos.count(cella) > 0) e, se si', se la
	# nave e' colpita o affondata.
	# Nella lista self.acqua sono contenute
	# le tuple che rappresentano le giocate in cui nessuna nave e' stata colpita.
	# Tutte le altre posizioni sono "mare".
	#
	def disegna(self):
		
		row = self.dimX - 1

		print("") # a capo per dare un po' di spazio.
		
		while row >= 0:
			# stampo il numero di riga seguito da un carattere di tabulazione per spaziare
			# i numeri di riga dall'inizio della scacchiera.
			print(row + 1, "\t|", end="")
			col = 0
			while col < self.dimY:
				cellaoccupata = False
				# creo tupla (row, col)
				tuplepos = (row, col)

				for nave in self.navi:
					affondata = nave.affondata()
					if nave.pos.count(tuplepos) > 0:
				
						if affondata:
							print("X", end="")
							cellaoccupata = True
		
						elif nave.colpita(row, col):
							print("+", end="")
							cellaoccupata = True

						elif self.naviVisibili:
							cellaoccupata = True
							if nave.orizzontale(): # rappresentata con "="
								print("=", end="")
							else:
								print("#", end="") # rappresentata con "#"
								
				# nave non colpita, ne' affondata. Se self.naviVisibili e' False, allora
				# disegno una 'o' se la tupla (row,col) e' in acqua, altrimenti disegno
				# o un'ondina oppure un semplice spazio, in modo casuale, per "disegnare"
				# il mare
				#
				if not cellaoccupata and tuplepos in self.acqua:
					print("o", end="")
				elif not cellaoccupata:
					# disegno onde, casualmente
					onda = randint(0,1)
					if onda == 1:
						print("~", end="")
					else:
						print(" ", end="")

				col = col + 1
			row = row  - 1
			print("") # a capo dopo una riga

		print("\t+", end="")
		
		# disegno il bordo inferiore della scacchiera
		for i in range(0, self.dimY):
			print("-", end="")

		print("\n\t ", end="") # a capo + tab + uno spazio
		
		# stampo le lettere della scacchiera
		for i in range(0, self.dimY):
			print(chr(ord('A') + i), end="")
		print("\n")

	def colpisci(self, x, y):
		colpita = False
		for nave in self.navi:
			if nave.colpisci(x, y):
				colpita = True
				break
		if not colpita:
			self.acqua.append((x,y))


	def tutteAffondate(self):
		for nave in self.navi:
			if not nave.affondata():
				return False
		return True 
